@@ -28,33 +28,29 @@
 #include <common/panel-debug.h>
 #include <common/panel-private.h>
 
-enum
+
+
+PanelDebugFlag panel_debug_flags = 0;
+
+
+
+/* additional debug levels */
+static const GDebugKey panel_debug_keys[] =
 {
-  DEBUG_LEVEL_UNKNOWN,
-  DEBUG_LEVEL_NONE,
-  DEBUG_LEVEL_ALL_DOMAINS,
-  DEBUG_LEVEL_FILTER_DOMAINS
+  { "gdb", PANEL_DEBUG_GDB }
 };
 
 
 
-gboolean panel_debug_enabled = FALSE;
-
-
-
 void
 panel_debug (const gchar *domain,
              const gchar *message,
              ...)
 {
-  static volatile gsize   level__volatile = DEBUG_LEVEL_UNKNOWN;
-  gsize                   level;
-  static gchar          **domains = NULL;
+  static volatile gsize   level__volatile = 0;
   const gchar            *value;
-  guint                   i;
-  gchar                  *string;
+  gchar                  *string, *path;
   va_list                 args;
-  gboolean                found;
 
   panel_return_if_fail (domain != NULL);
   panel_return_if_fail (message != NULL);
@@ -63,38 +59,40 @@ panel_debug (const gchar *domain,
   if (g_once_init_enter (&level__volatile))
     {
       value = g_getenv ("PANEL_DEBUG");
-      if (G_LIKELY (value == NULL))
+      if (G_UNLIKELY (value != NULL))
         {
-          level = DEBUG_LEVEL_NONE;
-        }
-      else if (strcmp (value, "1") == 0)
-        {
-          level = DEBUG_LEVEL_ALL_DOMAINS;
-        }
-      else
-        {
-          level = DEBUG_LEVEL_FILTER_DOMAINS;
-          domains = g_strsplit (value, ":", -1);
+          panel_debug_flags = g_parse_debug_string (value, panel_debug_keys,
+                                                    G_N_ELEMENTS (panel_debug_keys));
+
+          /* always enable debug logging */
+          PANEL_SET_FLAG (panel_debug_flags, PANEL_DEBUG_YES);
+
+          /* TODO: only print this in the main application */
+          if (PANEL_HAS_FLAG (panel_debug_flags, PANEL_DEBUG_GDB))
+            {
+              path = g_find_program_in_path ("gdb");
+              if (G_LIKELY (path != NULL))
+                {
+                  g_printerr (PACKAGE_NAME "(debug): running plugins with %s; "
+                              "log files stored in '%s'\n", path, g_get_tmp_dir ());
+                  g_free (path);
+                }
+              else
+                {
+                  PANEL_UNSET_FLAG (panel_debug_flags, PANEL_DEBUG_GDB);
+
+                  g_printerr (PACKAGE_NAME "(debug): gdb not found in PATH; mode disabled\n");
+                }
+            }
         }
 
-      panel_debug_enabled = (level != DEBUG_LEVEL_NONE);
-
-      g_once_init_leave (&level__volatile, level);
+      g_once_init_leave (&level__volatile, 1);
     }
 
   /* leave when debug is disabled */
-  if (level__volatile == DEBUG_LEVEL_NONE)
+  if (panel_debug_flags == 0)
     return;
 
-  if (level__volatile == DEBUG_LEVEL_FILTER_DOMAINS)
-    {
-      for (i = 0, found = FALSE; domains[i] != NULL && !found; i++)
-        if (strcmp (domains[i], domain) == 0)
-          found = TRUE;
-      if (!found)
-        return;
-    }
-
   va_start (args, message);
   string = g_strdup_vprintf (message, args);
   va_end (args);
