@@ -19,16 +19,25 @@
 #ifndef __PANEL_DEBUG_H__
 #define __PANEL_DEBUG_H__
 
+#define PANEL_DEBUG_DOMAIN_MAIN        "main"
 #define PANEL_DEBUG_DOMAIN_POSITIONING "positioning"
 #define PANEL_DEBUG_DOMAIN_STRUTS      "struts"
 #define PANEL_DEBUG_DOMAIN_APPLICATION "application"
 #define PANEL_DEBUG_DOMAIN_EXTERNAL    "external"
 #define PANEL_DEBUG_DOMAIN_EXTERNAL46  "external46"
 #define PANEL_DEBUG_DOMAIN_TASKLIST    "tasklist"
+#define PANEL_DEBUG_DOMAIN_BASE_WINDOW "base-window"
 
 #define PANEL_DEBUG_BOOL(bool) ((bool) ? "true" : "false")
 
-extern gboolean panel_debug_enabled;
+typedef enum
+{
+  PANEL_DEBUG_YES = 1 << 0, /* always enabled if PANEL_DEBUG is not %NULL */
+  PANEL_DEBUG_GDB = 1 << 1  /* run plugin through gdb */
+}
+PanelDebugFlag;
+
+extern PanelDebugFlag panel_debug_flags;
 
 void panel_debug (const gchar *domain,
                   const gchar *message,
