@@ -38,19 +38,6 @@ G_BEGIN_DECLS
  * Macros to register old external panel plugins that are compiled as executables.
  **/
 
-enum /*< skip >*/
-{
-  PANEL_CLIENT_EVENT_REMOVED,
-  PANEL_CLIENT_EVENT_SAVE,
-  PANEL_CLIENT_EVENT_SET_BACKGROUND_ALPHA,
-  PANEL_CLIENT_EVENT_SET_LOCKED,
-  PANEL_CLIENT_EVENT_SET_ORIENTATION,
-  PANEL_CLIENT_EVENT_SET_SCREEN_POSITION,
-  PANEL_CLIENT_EVENT_SET_SENSITIVE,
-  PANEL_CLIENT_EVENT_SET_SIZE,
-  PANEL_CLIENT_EVENT_SHOW_ABOUT,
-  PANEL_CLIENT_EVENT_SHOW_CONFIGURE
-};
 
 /*< private >*/
 #define _PANEL_CLIENT_EVENT_ATOM "XFCE4_PANEL_PLUGIN_46"
@@ -273,18 +260,35 @@ enum /*< skip >*/
  *                  for more information.
  **/
 #define XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL_FULL(construct_func, preinit_func, check_func) \
-  static GdkAtom  _xpp_atom = GDK_NONE; \
-  static gdouble  _xpp_alpha = 1.00; \
-  static gboolean _xpp_composited = FALSE; \
+  static GdkAtom          _xpp_atom = GDK_NONE; \
+  static gdouble          _xpp_alpha = 1.00; \
+  static guint            _xpp_bg_style = 0; \
+  static GdkColor         _xpp_bg_color = { 0, }; \
+  static const gchar     *_xpp_bg_image = NULL; \
+  static cairo_pattern_t *_xpp_bg_image_cache = NULL; \
+  static gboolean         _xpp_debug = FALSE; \
+  static gint             _xpp_retval = PLUGIN_EXIT_FAILURE; \
+  \
+  static void \
+  _xpp_quit_main_loop (void) \
+  { \
+     static gboolean quiting = FALSE; \
+     \
+     if (!quiting) \
+       { \
+         quiting = TRUE; \
+         gtk_main_quit (); \
+       } \
+  } \
   \
   static gboolean \
   _xpp_client_event (GtkWidget       *plug, \
                      GdkEventClient  *event, \
                      XfcePanelPlugin *xpp) \
   { \
-    XfcePanelPluginProvider *provider = XFCE_PANEL_PLUGIN_PROVIDER (xpp); \
-    gint                     value; \
-    gint                     message; \
+    XfcePanelPluginProvider          *provider = XFCE_PANEL_PLUGIN_PROVIDER (xpp); \
+    gint                              value; \
+    XfcePanelPluginProviderPropType   type; \
     \
     g_return_val_if_fail (XFCE_IS_PANEL_PLUGIN (xpp), TRUE); \
     g_return_val_if_fail (GTK_IS_PLUG (plug), TRUE); \
@@ -293,55 +297,74 @@ enum /*< skip >*/
     \
     if (event->message_type == _xpp_atom) \
       { \
-        message = event->data.s[0]; \
+        type = event->data.s[0]; \
         value = event->data.s[1]; \
         \
-        switch (message) \
+        switch (type) \
           { \
-          case PANEL_CLIENT_EVENT_REMOVED: \
+          case PROVIDER_PROP_TYPE_ACTION_REMOVED: \
             xfce_panel_plugin_provider_removed (provider); \
             break; \
             \
-          case PANEL_CLIENT_EVENT_SAVE: \
+          case PROVIDER_PROP_TYPE_ACTION_SAVE: \
             xfce_panel_plugin_provider_save (provider); \
             break; \
             \
-          case PANEL_CLIENT_EVENT_SET_BACKGROUND_ALPHA: \
+          case PROVIDER_PROP_TYPE_SET_BACKGROUND_ALPHA: \
             _xpp_alpha = value / 100.00; \
-            if (_xpp_composited) \
+            if (gtk_widget_is_composited (plug)) \
               gtk_widget_queue_draw (plug); \
             break; \
             \
-          case PANEL_CLIENT_EVENT_SET_LOCKED: \
+          case PROVIDER_PROP_TYPE_SET_LOCKED: \
             xfce_panel_plugin_provider_set_locked (provider, !!value); \
             break; \
             \
-          case PANEL_CLIENT_EVENT_SET_ORIENTATION: \
+          case PROVIDER_PROP_TYPE_SET_ORIENTATION: \
             xfce_panel_plugin_provider_set_orientation (provider, value); \
             break; \
             \
-          case PANEL_CLIENT_EVENT_SET_SCREEN_POSITION: \
+          case PROVIDER_PROP_TYPE_SET_SCREEN_POSITION: \
             xfce_panel_plugin_provider_set_screen_position (provider, value); \
             break; \
             \
-          case PANEL_CLIENT_EVENT_SET_SENSITIVE: \
+          case PROVIDER_PROP_TYPE_SET_SENSITIVE: \
             gtk_widget_set_sensitive (plug, !!value); \
             break; \
             \
-          case PANEL_CLIENT_EVENT_SET_SIZE: \
+          case PROVIDER_PROP_TYPE_SET_SIZE: \
             xfce_panel_plugin_provider_set_size (provider, value); \
             break; \
             \
-          case PANEL_CLIENT_EVENT_SHOW_ABOUT: \
+          case PROVIDER_PROP_TYPE_ACTION_SHOW_ABOUT: \
             xfce_panel_plugin_provider_show_about (provider); \
             break; \
             \
-          case PANEL_CLIENT_EVENT_SHOW_CONFIGURE: \
+          case PROVIDER_PROP_TYPE_ACTION_SHOW_CONFIGURE: \
             xfce_panel_plugin_provider_show_configure (provider); \
             break; \
             \
+          case PROVIDER_PROP_TYPE_ACTION_QUIT_FOR_RESTART: \
+            _xpp_retval = PLUGIN_EXIT_SUCCESS_AND_RESTART; \
+          case PROVIDER_PROP_TYPE_ACTION_QUIT: \
+            _xpp_quit_main_loop (); \
+            break; \
+            \
+          case PROVIDER_PROP_TYPE_SET_BACKGROUND_COLOR: \
+            _xpp_bg_color.red = event->data.s[1]; \
+            _xpp_bg_color.green = event->data.s[2]; \
+            _xpp_bg_color.blue = event->data.s[3]; \
+            _xpp_bg_style = 1; \
+            gtk_widget_queue_draw (plug); \
+            break; \
+            \
+          case PROVIDER_PROP_TYPE_ACTION_BACKGROUND_UNSET: \
+            _xpp_bg_style = 0; \
+            gtk_widget_queue_draw (plug); \
+            break; \
+            \
           default: \
-            g_warning ("Received unknow client event %d", message); \
+            g_warning ("Received unknow client event %u", type); \
             break; \
           } \
         \
@@ -362,10 +385,14 @@ enum /*< skip >*/
     \
     g_return_if_fail (GTK_IS_PLUG (plug)); \
     g_return_if_fail (XFCE_IS_PANEL_PLUGIN (xpp)); \
-    g_return_if_fail (GDK_IS_WINDOW (gtk_widget_get_window (plug))); \
+    g_return_if_fail (GTK_WIDGET_REALIZED (plug)); \
     g_return_if_fail (_xpp_atom != GDK_NONE); \
     g_return_if_fail (GTK_WIDGET_REALIZED (xpp)); \
     \
+    if (_xpp_debug) \
+      g_printerr ("xfce4-panel(%s): send provider signal %d\n", \
+                  xfce_panel_plugin_get_name (xpp), message); \
+    \
     event.type = GDK_CLIENT_EVENT; \
     event.window = gtk_widget_get_window (plug); \
     event.send_event = TRUE; \
@@ -386,13 +413,23 @@ enum /*< skip >*/
   } \
   \
   static void \
-  _xpp_realize (XfcePanelPlugin *xpp) \
+  _xpp_realize (XfcePanelPlugin *xpp, \
+                GtkPlug         *plug) \
   { \
     g_return_if_fail (XFCE_IS_PANEL_PLUGIN (xpp)); \
+    g_return_if_fail (GTK_IS_PLUG (plug)); \
+    g_return_if_fail (GTK_WIDGET_REALIZED (plug)); \
+    \
+    if (_xpp_debug) \
+      g_printerr ("xfce4-panel(%s): calling plugin construct function\n", \
+                  xfce_panel_plugin_get_name (xpp)); \
     \
     g_signal_handlers_disconnect_by_func (G_OBJECT (xpp), \
         G_CALLBACK (_xpp_realize), NULL); \
     \
+    g_signal_connect (G_OBJECT (xpp), "provider-signal", \
+        G_CALLBACK (_xpp_provider_signal), plug); \
+    \
     ((XfcePanelPluginFunc) construct_func) (xpp); \
   } \
   \
@@ -400,29 +437,78 @@ enum /*< skip >*/
   _xpp_expose_event (GtkWidget      *plug, \
                      GdkEventExpose *event) \
   { \
-    cairo_t  *cr; \
-    GdkColor *color; \
+    cairo_t        *cr; \
+    const GdkColor *color; \
+    gdouble         real_alpha; \
+    GdkPixbuf      *pixbuf; \
+    GError         *error = NULL; \
     \
-    if (_xpp_composited \
-        && GTK_WIDGET_DRAWABLE (plug) \
-        && _xpp_alpha < 1.00) \
+    if (!GTK_WIDGET_DRAWABLE (plug)) \
+      return FALSE; \
+    \
+    if (G_UNLIKELY (_xpp_bg_style == 2)) \
       { \
         cr = gdk_cairo_create (gtk_widget_get_window (plug)); \
         cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE); \
+        gdk_cairo_rectangle (cr, &event->area); \
+        cairo_clip (cr); \
         \
-        color = &(gtk_widget_get_style (plug)->bg[GTK_STATE_NORMAL]); \
-        cairo_set_source_rgba (cr, \
-                               color->red / 65535.00, \
-                               color->green / 65535.00, \
-                               color->blue / 65535.00, \
-                               _xpp_alpha); \
+        if (G_LIKELY (_xpp_bg_image_cache != NULL)) \
+          { \
+            cairo_set_source (cr, _xpp_bg_image_cache); \
+            cairo_paint (cr); \
+          } \
+        else \
+          { \
+            /* load the image in a pixbuf */ \
+            pixbuf = gdk_pixbuf_new_from_file (_xpp_bg_image, &error); \
+            if (G_LIKELY (pixbuf != NULL)) \
+              { \
+                gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0); \
+                g_object_unref (G_OBJECT (pixbuf)); \
+                \
+                _xpp_bg_image_cache = cairo_get_source (cr); \
+                cairo_pattern_reference (_xpp_bg_image_cache); \
+                cairo_pattern_set_extend (_xpp_bg_image_cache, CAIRO_EXTEND_REPEAT); \
+                cairo_paint (cr); \
+              } \
+            else \
+              { \
+                /* print error message */ \
+                g_warning ("Background image disabled, \"%s\" could not be loaded: %s", \
+                           _xpp_bg_image, error != NULL ? error->message : "No error"); \
+                g_error_free (error); \
+                \
+                /* disable background image */ \
+                _xpp_bg_style = 0; \
+              } \
+          } \
         \
-        cairo_rectangle (cr, event->area.x, event->area.y, \
-                         event->area.width, event->area.height); \
-        \
-        cairo_fill (cr); \
         cairo_destroy (cr); \
       } \
+    else \
+      { \
+        real_alpha = gtk_widget_is_composited (plug) ? _xpp_alpha : 1.00; \
+        \
+        if (_xpp_bg_style == 1 || real_alpha < 1.00) \
+          { \
+            if (G_LIKELY (_xpp_bg_style == 0)) \
+              color = &(gtk_widget_get_style (plug)->bg[GTK_STATE_NORMAL]); \
+            else \
+              color = &_xpp_bg_color; \
+            \
+            cr = gdk_cairo_create (gtk_widget_get_window (plug)); \
+            cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE); \
+            cairo_set_source_rgba (cr, \
+                                   color->red / 65535.00, \
+                                   color->green / 65535.00, \
+                                   color->blue / 65535.00, \
+                                   real_alpha); \
+            gdk_cairo_rectangle (cr, &event->area); \
+            cairo_fill (cr); \
+            cairo_destroy (cr); \
+          } \
+      } \
     \
     return FALSE; \
   } \
@@ -433,7 +519,7 @@ enum /*< skip >*/
     g_return_if_fail (GTK_IS_PLUG (plug)); \
     \
     if (!gtk_plug_get_embedded (plug)) \
-      gtk_main_quit (); \
+      _xpp_quit_main_loop (); \
   } \
   \
   gint \
@@ -445,11 +531,24 @@ enum /*< skip >*/
     gint             unique_id; \
     GdkNativeWindow  socket_id; \
     GdkColormap     *colormap = NULL; \
+    const gchar     *value; \
+    gchar           *base_name; \
+    \
+    value = g_getenv ("PANEL_DEBUG"); \
+    if (G_UNLIKELY (value != NULL)) \
+      { \
+        _xpp_debug = TRUE; \
+        \
+        base_name = g_filename_display_basename (argv[0]);\
+        g_printerr ("xfce4-panel(%s): compiled against libxfce4panel %s\n", \
+                    base_name, LIBXFCE4PANEL_VERSION); \
+        g_free (base_name); \
+      } \
     \
     if (G_UNLIKELY (argc < PLUGIN_ARGV_ARGUMENTS)) \
       { \
         g_critical ("Not enough arguments are passed to the plugin"); \
-        return PLUGIN_EXIT_FAILURE; \
+        return PLUGIN_EXIT_ARGUMENTS_FAILED; \
       } \
     \
     if (G_UNLIKELY (preinit_func != NULL)) \
@@ -480,14 +579,10 @@ enum /*< skip >*/
     gtk_widget_set_app_paintable (plug, TRUE); \
     \
     screen = gtk_widget_get_screen (plug); \
-    _xpp_composited = gtk_widget_is_composited (plug); \
-    if (_xpp_composited) \
-      colormap = gdk_screen_get_rgba_colormap (screen); \
+    \
+    colormap = gdk_screen_get_rgba_colormap (screen); \
     if (colormap == NULL) \
-      { \
-        colormap = gdk_screen_get_rgb_colormap (screen); \
-        _xpp_composited = FALSE; \
-      } \
+      colormap = gdk_screen_get_rgb_colormap (screen); \
     if (colormap != NULL) \
       gtk_widget_set_colormap (plug, colormap); \
     \
@@ -500,23 +595,33 @@ enum /*< skip >*/
                         "arguments", argv + PLUGIN_ARGV_ARGUMENTS, NULL); \
     gtk_container_add (GTK_CONTAINER (plug), xpp); \
     g_signal_connect_after (G_OBJECT (xpp), "realize", \
-        G_CALLBACK (_xpp_realize), NULL); \
+        G_CALLBACK (_xpp_realize), plug); \
     g_signal_connect_after (G_OBJECT (xpp), "destroy", \
-        G_CALLBACK (gtk_main_quit), NULL); \
-    g_signal_connect (G_OBJECT (xpp), "provider-signal", \
-        G_CALLBACK (_xpp_provider_signal), plug); \
+        G_CALLBACK (_xpp_quit_main_loop), NULL); \
     gtk_widget_show (xpp); \
     \
+    if (*argv[PLUGIN_ARGV_BACKGROUND_IMAGE] != '\0') \
+      { \
+        _xpp_bg_image = argv[PLUGIN_ARGV_BACKGROUND_IMAGE]; \
+        _xpp_bg_style = 2; \
+      } \
+    \
     g_signal_connect (G_OBJECT (plug), "client-event", \
        G_CALLBACK (_xpp_client_event), xpp); \
     gtk_widget_show (plug); \
     \
     gtk_main (); \
     \
+    if (_xpp_retval != PLUGIN_EXIT_SUCCESS_AND_RESTART) \
+      _xpp_retval = PLUGIN_EXIT_SUCCESS; \
+    \
+    if (_xpp_bg_image_cache != NULL) \
+      cairo_pattern_destroy (_xpp_bg_image_cache); \
+    \
     if (GTK_IS_WIDGET (plug)) \
       gtk_widget_destroy (plug); \
     \
-    return PLUGIN_EXIT_SUCCESS; \
+    return _xpp_retval; \
   }
 
 
