@@ -48,9 +48,6 @@
 #define XFCE_PANEL_PLUGIN_CONSTRUCTED(plugin) \
   PANEL_HAS_FLAG (XFCE_PANEL_PLUGIN (plugin)->priv->flags, \
                   PLUGIN_FLAG_CONSTRUCTED)
-#define XFCE_PANEL_PLUGIN_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
-                                            XFCE_TYPE_PANEL_PLUGIN, \
-                                            XfcePanelPluginPrivate))
 
 
 
@@ -60,6 +57,9 @@ typedef gint         (*ProviderToPluginInt)  (XfcePanelPluginProvider *provider)
 
 
 static void          xfce_panel_plugin_provider_init          (XfcePanelPluginProviderInterface *iface);
+static GObject      *xfce_panel_plugin_constructor            (GType                             type,
+                                                               guint                             n_props,
+                                                               GObjectConstructParam            *props);
 static void          xfce_panel_plugin_get_property           (GObject                          *object,
                                                                guint                             prop_id,
                                                                GValue                           *value,
@@ -138,9 +138,10 @@ typedef enum
 {
   PLUGIN_FLAG_DISPOSED       = 1 << 0,
   PLUGIN_FLAG_CONSTRUCTED    = 1 << 1,
-  PLUGIN_FLAG_SHOW_CONFIGURE = 1 << 2,
-  PLUGIN_FLAG_SHOW_ABOUT     = 1 << 3,
-  PLUGIN_FLAG_BLOCK_AUTOHIDE = 1 << 4
+  PLUGIN_FLAG_REALIZED       = 1 << 2,
+  PLUGIN_FLAG_SHOW_CONFIGURE = 1 << 3,
+  PLUGIN_FLAG_SHOW_ABOUT     = 1 << 4,
+  PLUGIN_FLAG_BLOCK_AUTOHIDE = 1 << 5
 }
 PluginFlags;
 
@@ -198,6 +199,7 @@ xfce_panel_plugin_class_init (XfcePanelPluginClass *klass)
   klass->construct = NULL;
 
   gobject_class = G_OBJECT_CLASS (klass);
+  gobject_class->constructor = xfce_panel_plugin_constructor;
   gobject_class->get_property = xfce_panel_plugin_get_property;
   gobject_class->set_property = xfce_panel_plugin_set_property;
   gobject_class->dispose = xfce_panel_plugin_dispose;
@@ -562,7 +564,7 @@ xfce_panel_plugin_class_init (XfcePanelPluginClass *klass)
 static void
 xfce_panel_plugin_init (XfcePanelPlugin *plugin)
 {
-  plugin->priv = XFCE_PANEL_PLUGIN_GET_PRIVATE (plugin);
+  plugin->priv = G_TYPE_INSTANCE_GET_PRIVATE (plugin, XFCE_TYPE_PANEL_PLUGIN, XfcePanelPluginPrivate);
 
   plugin->priv->name = NULL;
   plugin->priv->display_name = NULL;
@@ -607,6 +609,23 @@ xfce_panel_plugin_provider_init (XfcePanelPluginProviderInterface *iface)
 
 
 
+static GObject *
+xfce_panel_plugin_constructor (GType                  type,
+                               guint                  n_props,
+                               GObjectConstructParam *props)
+{
+  GObject *plugin;
+
+  plugin = G_OBJECT_CLASS (xfce_panel_plugin_parent_class)->constructor (type, n_props, props);
+
+  /* all the properties are set and can be used in public */
+  PANEL_SET_FLAG (XFCE_PANEL_PLUGIN (plugin)->priv->flags, PLUGIN_FLAG_CONSTRUCTED);
+
+  return plugin;
+}
+
+
+
 static void
 xfce_panel_plugin_get_property (GObject    *object,
                                 guint       prop_id,
@@ -770,30 +789,23 @@ xfce_panel_plugin_finalize (GObject *object)
 static void
 xfce_panel_plugin_realize (GtkWidget *widget)
 {
-  XfcePanelPluginClass *klass = NULL;
+  XfcePanelPluginClass *klass;
   XfcePanelPlugin      *plugin = XFCE_PANEL_PLUGIN (widget);
 
-  if (!PANEL_HAS_FLAG (plugin->priv->flags, PLUGIN_FLAG_CONSTRUCTED))
-    {
-      /* we use this flag to check plugin developers with gobject
-       * plugins, that they do not use plugin functions in the init
-       * stage. the properties are not set at that point an the
-       * panel is not aware of the signals and stuff */
-      PANEL_SET_FLAG (plugin->priv->flags, PLUGIN_FLAG_CONSTRUCTED);
-
-      /* wether this is an object plugin */
-      klass = XFCE_PANEL_PLUGIN_GET_CLASS (widget);
-      if (klass->construct == NULL)
-        klass = NULL;
-    }
-
-  /* let gtk to realize the plugin */
+  /* let gtk realize the plugin */
   (*GTK_WIDGET_CLASS (xfce_panel_plugin_parent_class)->realize) (widget);
 
-  /* check if there is a construct function attached to this plugin if
-   * so, run it */
-  if (klass != NULL)
-    (*klass->construct) (XFCE_PANEL_PLUGIN (widget));
+  /* launch the construct function for object oriented plugins, but
+   * do this only once */
+  if (!PANEL_HAS_FLAG (plugin->priv->flags, PLUGIN_FLAG_REALIZED))
+    {
+      PANEL_SET_FLAG (plugin->priv->flags, PLUGIN_FLAG_REALIZED);
+
+      /* whether this is an object plugin */
+      klass = XFCE_PANEL_PLUGIN_GET_CLASS (widget);
+      if (klass->construct != NULL)
+        (*klass->construct) (XFCE_PANEL_PLUGIN (widget));
+    }
 }
 
 
@@ -912,29 +924,15 @@ xfce_panel_plugin_menu_panel_preferences (XfcePanelPlugin *plugin)
 
 
 static void
-xfce_panel_plugin_menu_panel_quit (XfcePanelPlugin *plugin)
+xfce_panel_plugin_menu_panel_logout (XfcePanelPlugin *plugin)
 {
   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN (plugin));
   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (plugin));
   panel_return_if_fail (XFCE_PANEL_PLUGIN_CONSTRUCTED (plugin));
 
-  /* quit the panel/session */
+  /* logout the session */
   xfce_panel_plugin_provider_emit_signal (XFCE_PANEL_PLUGIN_PROVIDER (plugin),
-                                          PROVIDER_SIGNAL_PANEL_QUIT);
-}
-
-
-
-static void
-xfce_panel_plugin_menu_panel_restart (XfcePanelPlugin *plugin)
-{
-  panel_return_if_fail (XFCE_IS_PANEL_PLUGIN (plugin));
-  panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (plugin));
-  panel_return_if_fail (XFCE_PANEL_PLUGIN_CONSTRUCTED (plugin));
-
-  /* restart the panel */
-  xfce_panel_plugin_provider_emit_signal (XFCE_PANEL_PLUGIN_PROVIDER (plugin),
-                                          PROVIDER_SIGNAL_PANEL_RESTART);
+                                          PROVIDER_SIGNAL_PANEL_LOGOUT);
 }
 
 
@@ -1106,21 +1104,14 @@ xfce_panel_plugin_menu_get (XfcePanelPlugin *plugin)
           gtk_widget_show (item);
         }
 
-      /* quit item */
-      item = gtk_image_menu_item_new_from_stock (GTK_STOCK_QUIT, NULL);
+      /* logout item */
+      item = gtk_image_menu_item_new_with_mnemonic (_("Log _Out"));
       g_signal_connect_swapped (G_OBJECT (item), "activate",
-          G_CALLBACK (xfce_panel_plugin_menu_panel_quit), plugin);
+          G_CALLBACK (xfce_panel_plugin_menu_panel_logout), plugin);
       gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
       gtk_widget_show (item);
 
-      /* restart item */
-      item = gtk_image_menu_item_new_with_mnemonic (_("_Restart"));
-      g_signal_connect_swapped (G_OBJECT (item), "activate",
-          G_CALLBACK (xfce_panel_plugin_menu_panel_restart), plugin);
-      gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
-      gtk_widget_show (item);
-
-      image = gtk_image_new_from_stock (GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU);
+      image = gtk_image_new_from_icon_name ("system-log-out", GTK_ICON_SIZE_MENU);
       gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
       gtk_widget_show (image);
 
