@@ -38,6 +38,8 @@
 #include <libxfce4panel/xfce-panel-plugin-provider.h>
 
 #include <panel/panel-dbus-service.h>
+#include <panel/panel-base-window.h>
+#include <panel/panel-plugin-external-46.h>
 #include <panel/panel-window.h>
 #include <panel/panel-application.h>
 #include <panel/panel-itembar.h>
@@ -268,6 +270,9 @@ panel_application_xfconf_window_bindings (PanelApplication *application,
     { "enter-opacity", G_TYPE_UINT },
     { "leave-opacity", G_TYPE_UINT },
     { "background-alpha", G_TYPE_UINT },
+    { "background-style", G_TYPE_UINT },
+    { "background-color", GDK_TYPE_COLOR },
+    { "background-image", G_TYPE_STRING },
     { "output-name", G_TYPE_STRING },
     { "position", G_TYPE_STRING },
     { "disable-struts", G_TYPE_BOOLEAN },
@@ -545,10 +550,9 @@ panel_application_plugin_provider_signal (XfcePanelPluginProvider       *provide
       panel_preferences_dialog_show (window);
       break;
 
-    case PROVIDER_SIGNAL_PANEL_QUIT:
-    case PROVIDER_SIGNAL_PANEL_RESTART:
-      /* quit or restart */
-      panel_dbus_service_exit_panel (provider_signal == PROVIDER_SIGNAL_PANEL_RESTART);
+    case PROVIDER_SIGNAL_PANEL_LOGOUT:
+      /* logout */
+      panel_application_logout ();
       break;
 
     case PROVIDER_SIGNAL_PANEL_ABOUT:
@@ -616,6 +620,13 @@ panel_application_plugin_insert (PanelApplication  *application,
   g_signal_connect (G_OBJECT (provider), "provider-signal",
       G_CALLBACK (panel_application_plugin_provider_signal), application);
 
+  /* work around the problem that we need a background before
+   * realizing for 4.6 panel plugins */
+  if (PANEL_BASE_WINDOW (window)->background_style == PANEL_BG_STYLE_IMAGE
+      && PANEL_IS_PLUGIN_EXTERNAL_46 (provider))
+    panel_plugin_external_set_background_image (PANEL_PLUGIN_EXTERNAL (provider),
+        PANEL_BASE_WINDOW (window)->background_image);
+
   /* add the item to the panel */
   itembar = gtk_bin_get_child (GTK_BIN (window));
   panel_itembar_insert (PANEL_ITEMBAR (itembar),
@@ -1239,9 +1250,6 @@ panel_application_new_window (PanelApplication *application,
   /* create panel window */
   window = panel_window_new ();
 
-  /* realize, else some properties do not apply */
-  gtk_widget_realize (window);
-
   /* monitor window destruction */
   g_signal_connect (G_OBJECT (window), "destroy",
       G_CALLBACK (panel_application_window_destroyed), application);
@@ -1397,3 +1405,25 @@ panel_application_get_locked (PanelApplication *application)
    * locked but maybe not all the plugins) */
   return TRUE;
 }
+
+
+
+void
+panel_application_logout (void)
+{
+  XfceSMClient *sm_client;
+  GError       *error = NULL;
+
+  /* first try to session client to logout else fallback and spawn xfce4-session-logout */
+  sm_client = xfce_sm_client_get ();
+  if (xfce_sm_client_is_connected (sm_client))
+    {
+      xfce_sm_client_request_shutdown (sm_client, XFCE_SM_CLIENT_SHUTDOWN_HINT_ASK);
+    }
+  else if (!g_spawn_command_line_async ("xfce4-session-logout", &error))
+    {
+      xfce_dialog_show_error (NULL, error, _("Failed to execute command \"%s\""),
+                              "xfce4-session-logout");
+      g_error_free (error);
+    }
+}
