@@ -109,6 +109,7 @@ static void         panel_window_screen_changed             (GtkWidget        *w
                                                              GdkScreen        *previous_screen);
 static void         panel_window_style_set                  (GtkWidget        *widget,
                                                              GtkStyle         *previous_style);
+static void         panel_window_realize                    (GtkWidget        *widget);
 static StrutsEgde   panel_window_screen_struts_edge         (PanelWindow      *window);
 static void         panel_window_screen_struts_set          (PanelWindow      *window);
 static void         panel_window_screen_force_update        (PanelWindow      *window);
@@ -324,6 +325,7 @@ panel_window_class_init (PanelWindowClass *klass)
   gtkwidget_class->size_allocate = panel_window_size_allocate;
   gtkwidget_class->screen_changed = panel_window_screen_changed;
   gtkwidget_class->style_set = panel_window_style_set;
+  gtkwidget_class->realize = panel_window_realize;
 
   g_object_class_install_property (gobject_class,
                                    PROP_HORIZONTAL,
@@ -1307,6 +1309,20 @@ panel_window_style_set (GtkWidget *widget,
 
 
 
+static void
+panel_window_realize (GtkWidget *widget)
+{
+  PanelWindow *window = PANEL_WINDOW (widget);
+
+  (*GTK_WIDGET_CLASS (panel_window_parent_class)->realize) (widget);
+
+  /* set struts if we snap to an edge */
+  if (window->struts_edge != STRUTS_EDGE_NONE)
+    panel_window_screen_struts_set (window);
+}
+
+
+
 static StrutsEgde
 panel_window_screen_struts_edge (PanelWindow *window)
 {
@@ -1370,10 +1386,12 @@ panel_window_screen_struts_set (PanelWindow *window)
   const gchar   *strut_xy[] = { "y", "y", "x", "x" };
 
   panel_return_if_fail (PANEL_IS_WINDOW (window));
-  panel_return_if_fail (GDK_IS_WINDOW (GTK_WIDGET (window)->window));
   panel_return_if_fail (cardinal_atom != 0 && net_wm_strut_partial_atom != 0);
   panel_return_if_fail (GDK_IS_SCREEN (window->screen));
 
+  if (!GTK_WIDGET_REALIZED (window))
+    return;
+
   /* set the struts */
   /* note that struts are relative to the screen edge! */
   if (window->struts_edge == STRUTS_EDGE_TOP)
@@ -1425,6 +1443,7 @@ panel_window_screen_struts_set (PanelWindow *window)
   gdk_error_trap_push ();
 
   /* set the wm strut partial */
+  panel_return_if_fail (GDK_IS_WINDOW (GTK_WIDGET (window)->window));
   gdk_property_change (GTK_WIDGET (window)->window,
                        net_wm_strut_partial_atom,
                        cardinal_atom, 32, GDK_PROP_MODE_REPLACE,
@@ -1442,7 +1461,7 @@ panel_window_screen_struts_set (PanelWindow *window)
   if (gdk_error_trap_pop () != 0)
     g_critical ("Failed to set the struts");
 
-  if (panel_debug_enabled)
+  if (G_UNLIKELY (panel_debug_flags != 0))
     {
       if (struts[STRUT_LEFT] != 0)
         n = STRUT_LEFT;
@@ -1972,7 +1991,8 @@ panel_window_set_autohide (PanelWindow *window,
   GtkWidget   *popup;
   guint        i;
   const gchar *properties[] = { "enter-opacity", "leave-opacity",
-                                "background-alpha", "borders" };
+                                "background-alpha", "borders",
+                                "background-style", "background-color" };
 
   panel_return_if_fail (PANEL_IS_WINDOW (window));
 
@@ -2040,14 +2060,6 @@ panel_window_set_autohide (PanelWindow *window,
 
 
 
-static void
-panel_window_menu_quit (gpointer boolean)
-{
-  panel_dbus_service_exit_panel (GPOINTER_TO_UINT (boolean) == 1);
-}
-
-
-
 static void
 panel_window_menu_help (void)
 {
@@ -2128,21 +2140,14 @@ panel_window_menu_popup (PanelWindow *window,
       gtk_widget_show (item);
     }
 
-  /* quit item */
-  item = gtk_image_menu_item_new_from_stock (GTK_STOCK_QUIT, NULL);
+  /* logout item */
+  item = gtk_image_menu_item_new_with_mnemonic (_("Log _Out"));
   g_signal_connect_swapped (G_OBJECT (item), "activate",
-      G_CALLBACK (panel_window_menu_quit), GUINT_TO_POINTER (0));
+      G_CALLBACK (panel_application_logout), NULL);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
   gtk_widget_show (item);
 
-  /* restart item */
-  item = gtk_image_menu_item_new_with_mnemonic (_("_Restart"));
-  g_signal_connect_swapped (G_OBJECT (item), "activate",
-      G_CALLBACK (panel_window_menu_quit), GUINT_TO_POINTER (1));
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-  gtk_widget_show (item);
-
-  image = gtk_image_new_from_stock (GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU);
+  image = gtk_image_new_from_icon_name ("system-log-out", GTK_ICON_SIZE_MENU);
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
   gtk_widget_show (image);
 
@@ -2326,20 +2331,32 @@ void
 panel_window_set_povider_info (PanelWindow *window,
                                GtkWidget   *provider)
 {
+  PanelBaseWindow *base_window = PANEL_BASE_WINDOW (window);
+
   panel_return_if_fail (PANEL_IS_WINDOW (window));
   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (provider));
 
   xfce_panel_plugin_provider_set_locked (XFCE_PANEL_PLUGIN_PROVIDER (provider),
                                          panel_window_get_locked (window));
 
-  if (PANEL_BASE_WINDOW (window)->background_alpha < 1.0)
+  if (base_window->background_alpha < 1.0)
     {
       if (PANEL_IS_PLUGIN_EXTERNAL (provider))
         panel_plugin_external_set_background_alpha (PANEL_PLUGIN_EXTERNAL (provider),
-            PANEL_BASE_WINDOW (window)->background_alpha);
-      else if (PANEL_IS_PLUGIN_EXTERNAL_46 (provider))
-        panel_plugin_external_46_set_background_alpha (PANEL_PLUGIN_EXTERNAL_46 (provider),
-            PANEL_BASE_WINDOW (window)->background_alpha);
+            base_window->background_alpha);
+    }
+
+  if (base_window->background_style == PANEL_BG_STYLE_COLOR)
+    {
+      if (PANEL_IS_PLUGIN_EXTERNAL (provider))
+        panel_plugin_external_set_background_color (PANEL_PLUGIN_EXTERNAL (provider),
+            base_window->background_color);
+    }
+  else if (base_window->background_style == PANEL_BG_STYLE_IMAGE)
+    {
+      if (PANEL_IS_PLUGIN_EXTERNAL (provider))
+        panel_plugin_external_set_background_image (PANEL_PLUGIN_EXTERNAL (provider),
+            base_window->background_image);
     }
 
   panel_window_set_plugin_orientation (provider, window);
