@@ -56,6 +56,8 @@ static void     pager_plugin_construct                 (XfcePanelPlugin   *panel
 static void     pager_plugin_free_data                 (XfcePanelPlugin   *panel_plugin);
 static gboolean pager_plugin_size_changed              (XfcePanelPlugin   *panel_plugin,
                                                         gint               size);
+static void     pager_plugin_orientation_changed       (XfcePanelPlugin   *panel_plugin,
+                                                        GtkOrientation     orientation);
 static void     pager_plugin_configure_plugin          (XfcePanelPlugin   *panel_plugin);
 
 
@@ -112,6 +114,7 @@ pager_plugin_class_init (PagerPluginClass *klass)
   plugin_class->construct = pager_plugin_construct;
   plugin_class->free_data = pager_plugin_free_data;
   plugin_class->size_changed = pager_plugin_size_changed;
+  plugin_class->orientation_changed = pager_plugin_orientation_changed;
   plugin_class->configure_plugin = pager_plugin_configure_plugin;
 
   g_object_class_install_property (gobject_class,
@@ -226,9 +229,11 @@ static gboolean
 pager_plugin_scroll_event (GtkWidget      *widget,
                            GdkEventScroll *event)
 {
-  PagerPlugin         *plugin = XFCE_PAGER_PLUGIN (widget);
-  WnckWorkspace       *active, *neighbor;
-  WnckMotionDirection  direction;
+  PagerPlugin   *plugin = XFCE_PAGER_PLUGIN (widget);
+  WnckWorkspace *active_ws;
+  WnckWorkspace *new_ws;
+  gint           active_n;
+  gint           n_workspaces;
 
   panel_return_val_if_fail (WNCK_IS_SCREEN (plugin->wnck_screen), FALSE);
 
@@ -236,33 +241,25 @@ pager_plugin_scroll_event (GtkWidget      *widget,
   if (plugin->scrolling == FALSE)
     return TRUE;
 
-  /* translate the gdk scroll direction into a wnck motion direction */
-  switch (event->direction)
-    {
-    case GDK_SCROLL_UP:
-      direction = WNCK_MOTION_UP;
-      break;
+  active_ws = wnck_screen_get_active_workspace (plugin->wnck_screen);
+  active_n = wnck_workspace_get_number (active_ws);
 
-    case GDK_SCROLL_DOWN:
-      direction = WNCK_MOTION_DOWN;
-      break;
+  if (event->direction == GDK_SCROLL_UP
+      || event->direction == GDK_SCROLL_LEFT)
+    active_n--;
+  else
+    active_n++;
 
-    case GDK_SCROLL_LEFT:
-      direction = WNCK_MOTION_LEFT;
-      break;
+  /* wrap around */
+  n_workspaces = wnck_screen_get_workspace_count (plugin->wnck_screen) - 1;
+  if (active_n < 0)
+    active_n = n_workspaces;
+  else if (active_n > n_workspaces)
+    active_n = 0;
 
-    default:
-      direction = WNCK_MOTION_RIGHT;
-      break;
-    }
-
-  /* get the active workspace's neighbor */
-  active = wnck_screen_get_active_workspace (plugin->wnck_screen);
-  neighbor = wnck_workspace_get_neighbor (active, direction);
-
-  /* if there is a neighbor, move to it */
-  if (neighbor != NULL)
-    wnck_workspace_activate (neighbor, event->time);
+  new_ws = wnck_screen_get_workspace (plugin->wnck_screen, active_n);
+  if (new_ws != NULL && active_ws != new_ws)
+    wnck_workspace_activate (new_ws, event->time);
 
   return TRUE;
 }
@@ -374,17 +371,23 @@ static gboolean
 pager_plugin_size_changed (XfcePanelPlugin *panel_plugin,
                            gint             size)
 {
-  if (xfce_panel_plugin_get_orientation (panel_plugin) ==
-      GTK_ORIENTATION_HORIZONTAL)
-    gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), -1, size);
-  else
-    gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), size, -1);
-
+  /* do not set fixed size */
   return TRUE;
 }
 
 
 
+static void
+pager_plugin_orientation_changed (XfcePanelPlugin *panel_plugin,
+                                  GtkOrientation   orientation)
+{
+  PagerPlugin *plugin = XFCE_PAGER_PLUGIN (panel_plugin);
+
+  wnck_pager_set_orientation (WNCK_PAGER (plugin->wnck_pager), orientation);
+}
+
+
+
 static void
 pager_plugin_configure_workspace_settings (GtkWidget *button)
 {
